#include <stdio.h>
#include <stdlib.h>
#include <alloc.h>
#include <math.h>

FILE *src, *dest, *bin;
char LastRead;
int curptr, outptr;

typedef struct {
  char          RIFF[4];
  long unsigned Len1;
  char          WAVEfmt[8];
  long unsigned lUnit;
  unsigned      Unit;
  unsigned      chan;
  long unsigned Hz;
  long unsigned BytePerSec;
  unsigned      BytePerSample;
  unsigned      Bits;
  char          data[4];
  long unsigned Len;
} WAVhdr;

WAVhdr whdr;

int GetNextHalfWave( int &hwlen, int &hwh ) {
  int cnt, umax, umin, ph;
  char chr;
  umax = umin = chr = LastRead;
  ph = (chr >= 128);
  cnt = 1;
  while (!feof(src) && ph == ((chr = fgetc(src)) >= 128) ) {
    cnt++;
    umax = (umax<=chr)?chr:umax;
    umin = (umin>=chr)?chr:umin;
  };
  hwh   = (ph?umax:umin);
  hwlen = cnt;
  LastRead = chr;
  return !feof(src);
}; // GetNextHalfWave //

int Lens[18];
int Hs[18];
int Bits[18];
unsigned THRESH = 11;  ///!!!

int FirstGuess( int B, int A ) {
//   return (abs(H-128)*L > THRESH ) ? 1 : 0;         //!!!!
  /*
  static int AA,lA,lB;
  AA = abs(128-A);
  lA = -(AA < 90) - (AA < 100) + 2;     // levels 0, 1, 2
  lB = -( B < 35) - ( B < 55 ) + 2;     // levels 0, 1, 2

  if (lA+lB<2) return 0;
  if (lA+lB>2) return 1;

  return (abs(A-128)*B > THRESH ) ? 1 : 0;
  */
  return (B>THRESH);
};

int RefillWaveBuf( void ) {
  int i, retflag;
  retflag = 1;
  if (curptr == -1) { // Total fill
    for (i=0; i<16; i++) {
      retflag = GetNextHalfWave( Lens[i], Hs[i] );
      Bits[i] = FirstGuess( Lens[i], Hs[i] );
    };
    curptr = 3;     // first 3 bits not considered !
    outptr = 0;
  } else {            // Partial refill
    if ( curptr > 12 ) {
      for (i=0; curptr+i-3<16; i++) {
        Lens[i] = Lens[curptr+i-3];
        Hs[i]   = Hs[curptr+i-3];
        Bits[i] = Bits[curptr+i-3];
      };
      for ( ; i<16; i++) { // i=i
        retflag = GetNextHalfWave( Lens[i], Hs[i] );
        Bits[i] = FirstGuess( Lens[i], Hs[i] );
      };
    curptr = 3;     // first 3 bits not considered !
    outptr = 0;
    };
  }; // else - no need to Refill
  curptr++;
  return retflag;
} // RefillWaveBuf //

void main( int _argc, char *_argv[] ) {
   int i;
   printf("\nĿ");
   printf("\n Wav -> Bin                                 ");
   printf("\n");
   if (_argc != 4 && _argc != 3) {
     printf("\n\n Usage:\n mikr <source> <destination> [<THRESH>]\n\n");
     exit(1);
   };
   if (_argc >= 3) {
     if ((src = fopen(_argv[1], "rb")) == NULL) {
       fprintf(stderr, "Cannot open source file.\n");
       exit(1);
     };
     if ((dest = fopen("nul", "wt")) == NULL) {
       fprintf(stderr, "Cannot open nul file.\n");
       exit(1);
     };
     if ((bin = fopen(_argv[2], "wb")) == NULL) {
       fprintf(stderr, "Cannot open destination file.\n");
       exit(1);
     };
   };

  fread (&whdr, sizeof(whdr), 1, src);
  LastRead = 0;

  if (_argc == 4 ) {
    sscanf(_argv[3],"%i",&THRESH);
    if ( THRESH<=2 || THRESH>100 ) THRESH = (unsigned int)(whdr.Hz/1000);
  } else {
    THRESH = (unsigned int)(whdr.Hz/1000);
  };
  printf("\nTHRESH = %i\n",THRESH);

  curptr = -1;

  int A,B,C,oldC=0,D,E,F,oldF=0,G,oldG=-1;
  int parity=0, bytelen=0;
  int mode=0; // no E6 yet
  char curbyte = 0;
  int bitcnt = 0;
  int bytecnt = 0;
  int curb;
  while ( RefillWaveBuf() ) {
    A = Hs[curptr];
    B = Lens[curptr];
    C = Bits[curptr];       /// !!

    if (parity !=0 && C==1) (Bits[curptr]=C=0),(mode?printf("## Parity! ##"):0);  // a law!
    parity = (C==1) ? 0 : 1-parity;
    F = (oldF == C) ? 0 : 1;
    G = (C==1 || oldC!=0 || oldG==-1) ? F : -1;
    bytelen += B;
    // (mode)?printf (" %4i; %2i;  %3i,%3i,%1i,,,%1i,%1i",curbyte,bitcnt,A,B,C,F,G):0;
    bytelen=0;
    if (G!=-1) {
      bitcnt++;
      curbyte = (curbyte << 1) + G;
      curb = curbyte;                          //
      if ( (curbyte == 230 || curbyte == 25) && mode == 0) {
        mode = 1;
        bitcnt = 0;
        if (curbyte != 230) F = 1-F;
        //printf("\n---------------------");
        bytelen=0;
      }
      else if ( mode != 0 && bitcnt == 8 ) {
        curb = curbyte;
        fprintf(dest,"%i: %c, %i, %X\n",bytecnt, curbyte, curb, curb);
         //printf(     "-----------%i: %c, %i, %X,   %i\n",
         //       bytecnt, curbyte, curb, curb, bytelen);
        fputc(curbyte,bin);
        bitcnt = 0;
        bytelen=0;
        bytecnt++;
      };
    };
    oldC = C;
    oldF = F;
    oldG = G;
  };

  fclose(src);
  fclose(dest);
  fclose(bin);


}

